package com.xhkjedu.sexam.service.notice;

import com.xhkjedu.sexam.mapper.exam.EClassMapper;
import com.xhkjedu.sexam.mapper.notice.*;
import com.xhkjedu.sexam.mapper.system.UserMapper;
import com.xhkjedu.sexam.model.notice.*;
import com.xhkjedu.sexam.utils.ExamUtil;
import com.xhkjedu.sexam.utils.SMSSender;
import com.xhkjedu.sexam.vo.exam.ExamVo;
import com.xhkjedu.sexam.vo.exam.ZtMsgVo;
import com.xhkjedu.sexam.vo.system.UserVo;
import com.xhkjedu.utils.N_Utils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.ArrayList;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * @author ywx
 * @classname NoticeService
 * @description
 * @date 2020/6/12 17:55
 **/
@Service
@Slf4j
public class NoticeService {
    @Resource
    private NoticeMapper noticeMapper;
    @Resource
    private NoticeTeacherMapper noticeTeacherMapper;
    @Resource
    private NoticeClassMapper noticeClassMapper;
    @Resource
    private NoticeSchoolMapper noticeSchoolMapper;
    @Resource
    private MsgMapper msgMapper;
    @Resource
    private UserMapper userMapper;
    @Resource
    private EClassMapper eClassMapper;

    //批量添加通知和短信
    @Async("asyncPoolTaskExecutor")
    public void batchSave(List<TNotice> noticeList, List<TMsg> msgList, Integer schoolid, Integer createid, Integer noticetype) {
        try {
            Integer createtime = N_Utils.getSecondTimestamp();
            if (N_Utils.isListNotEmpty(noticeList)) {//通知
                noticeList = noticeList.stream().distinct().collect(Collectors.toList());
                noticeList.forEach(n -> {
                    n.setNoticetxt(n.getNoticecontent().replaceAll("<[.[^>]]*>", ""));
                    n.setNoticetype(noticetype);
                    n.setCreatetime(createtime);
                    n.setCreateid(createid);
                    n.setSchoolid(schoolid);
                });
                noticeMapper.insertList(noticeList);

                List<TNoticeTeacher> teacherList = new ArrayList<>();
                List<TNoticeClass> classList = new ArrayList<>();
                List<TNoticeSchool> schoolList = new ArrayList<>();
                noticeList.forEach(n -> {
                    Integer noticeid = n.getId();
                    List<TNoticeTeacher> teachers = n.getTeachers().stream().distinct().collect(Collectors.toList());
                    teachers.forEach(t -> {
                        t.setNoticeid(noticeid);
                        t.setReaded(0);
                        t.setCreatetime(createtime);
                        t.setSchoolid(schoolid);
                    });
                    teacherList.addAll(teachers);

                    List<TNoticeClass> classes = n.getClasses();
                    classes.forEach(c -> {
                        c.setNoticeid(noticeid);
                        c.setCreateid(createid);
                        c.setCreatetime(createtime);
                    });
                    classList.addAll(classes);

                    TNoticeSchool ns = new TNoticeSchool();
                    ns.setNoticeid(noticeid);
                    ns.setSchoolid(schoolid);
                    ns.setCreateid(createid);
                    ns.setCreatetime(createtime);
                    schoolList.add(ns);
                });

                if (N_Utils.isListNotEmpty(teacherList)) noticeTeacherMapper.insertList(teacherList);//通知教师关联信息
                if (N_Utils.isListNotEmpty(classList)) noticeClassMapper.insertList(classList);//通知班级关联信息
                if (N_Utils.isListNotEmpty(schoolList)) noticeSchoolMapper.insertList(schoolList);//通知学校关联信息
            }

            if (N_Utils.isListNotEmpty(msgList)) {//短信
                msgList = msgList.stream().distinct().collect(Collectors.toList());
                List<TMsg> msgs = new ArrayList<>();
                if (noticetype == 1) {//通知对象1班级
                    for (TMsg msg : msgList) {
                        msg.setCreatetime(createtime);
                        msgs.add(msg);
                    }
                } else {//通知对象2老师
                    List<Integer> userids = msgList.stream().map(m -> m.getUserid()).distinct().collect(Collectors.toList());
                    List<UserVo> users = userMapper.listPhoneByUserIds(userids);//根据用户ids查找手机号
                    Map<Integer, TMsg> msgMap = msgList.stream().collect(Collectors.toMap(m -> m.getUserid(), m -> m));
                    for (int i = 0; i < users.size(); i++) {
                        UserVo u = users.get(i);
                        String userphone = u.getUserphone();
                        if (N_Utils.isEmpty(userphone)) continue;
                        TMsg m = msgMap.get(u.getUserid());
                        m.setUserphone(userphone);
                        m.setCreatetime(createtime);
                        msgs.add(m);
                    }
                }
                if (N_Utils.isListNotEmpty(msgs)) {
                    msgMapper.insertList(msgs);//短信保存到数据库

                    //发短信
                    Integer pageSize = 2000;//手机号，多个手机号为用半角 , 分开，如13899999999,13688888888(最多2000个，必填)
                    Map<String, List<TMsg>> mcollect = msgs.stream().collect(Collectors.groupingBy(s -> s.getMsgtxt()
                            , LinkedHashMap::new, Collectors.toList()));
                    for (Map.Entry<String, List<TMsg>> entry : mcollect.entrySet()) {
                        List<TMsg> list = entry.getValue();
                        String msgtxt = list.get(0).getMsgtxt();
                        for (int i = 0; i < list.size(); i += pageSize) {
                            ZtMsgVo vo = new ZtMsgVo();
                            String userphone = list.stream().skip(i).limit(pageSize)
                                    .map(s -> s.getUserphone()).collect(Collectors.joining(","));
                            vo.setMobile(userphone);
                            vo.setContent(msgtxt);
                            SMSSender.sendSms(vo);
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("批量添加通知和短信失败", e.getMessage());
        }
    }

    //批量添加通知和短信（发布考试用）
    @Async("asyncPoolTaskExecutor")
    public void batchSave2(ExamVo examVo, List<UserVo> students) {
        Integer notice = examVo.getNotice();//是否发送通知1发送2不发送
        Integer msg = examVo.getMsg();//是否发送短信1发送2不发送
        String examcomm = examVo.getExamcomm();
        Integer createid = examVo.getHandleid();
        Integer examid = examVo.getExamid();
        String examname = examVo.getExamname();
        Integer schoolid = examVo.getSchoolid();
        try {
            List<TNotice> noticeList = new ArrayList<>();
            List<TNoticeClass> classList = new ArrayList<>();
            List<TNoticeTeacher> teacherList = new ArrayList<>();
            List<TMsg> msgList = new ArrayList<>();
            String noticetitle = examname + "-考试公告";
            List<Map> classes = eClassMapper.listByExamId(examid);
            String classids = classes.stream().map(c -> c.get("classid").toString()).collect(Collectors.joining(","));
            List<UserVo> teachers = userMapper.listTeacherByClassIds(classids);//获取考试班级任教老师
            if (notice == 1) {
                for (Map s : classes) {
                    TNoticeClass nt = new TNoticeClass();
                    nt.setClassid((Integer) s.get("classid"));
                    classList.add(nt);
                }
                TNotice n = new TNotice();
                n.setNoticetitle(noticetitle);
                n.setNoticecontent(examcomm);
                n.setNoticetype(1);
                n.setClasses(classList);
                noticeList.add(n);

                for (UserVo t : teachers) {
                    TNoticeTeacher nt = new TNoticeTeacher();
                    nt.setTeacherid(t.getUserid());
                    teacherList.add(nt);
                }
                TNotice n2 = new TNotice();
                n2.setNoticetitle(noticetitle);
                n2.setNoticecontent(examcomm);
                n2.setNoticetype(2);
                n2.setTeachers(teacherList);
                noticeList.add(n2);
            }
            if (msg == 1) {
                String msgtxt = noticetitle + "\n" + ExamUtil.htmlRemoveTag(examcomm);
                List<String> userphones = students.stream().filter(s -> N_Utils.isNotEmpty(s.getUserphone()))
                        .map(s -> s.getUserphone()).collect(Collectors.toList());
                for (String userphone : userphones) {
                    TMsg m = new TMsg();
                    m.setMsgtxt(msgtxt);
                    m.setUserphone(userphone);
                    msgList.add(m);
                }
                userphones = teachers.stream().filter(s -> N_Utils.isNotEmpty(s.getUserphone()))
                        .map(s -> s.getUserphone()).collect(Collectors.toList());
                for (String userphone : userphones) {
                    TMsg m = new TMsg();
                    m.setMsgtxt(msgtxt);
                    m.setUserphone(userphone);
                    msgList.add(m);
                }
            }
            Integer createtime = N_Utils.getSecondTimestamp();
            if (N_Utils.isListNotEmpty(noticeList)) {//通知
                noticeList.forEach(n -> {
                    n.setNoticetxt(n.getNoticecontent().replaceAll("<[.[^>]]*>", ""));
                    n.setCreatetime(createtime);
                    n.setCreateid(createid);
                    n.setSchoolid(schoolid);
                });
                noticeMapper.insertList(noticeList);

                List<TNoticeTeacher> nteacherList = new ArrayList<>();
                List<TNoticeClass> nclassList = new ArrayList<>();
                List<TNoticeSchool> schoolList = new ArrayList<>();
                noticeList.forEach(n -> {
                    Integer noticeid = n.getId();
                    List<TNoticeTeacher> cteachers = n.getTeachers().stream().distinct().collect(Collectors.toList());
                    cteachers.forEach(t -> {
                        t.setNoticeid(noticeid);
                        t.setReaded(0);
                        t.setCreatetime(createtime);
                        t.setSchoolid(schoolid);
                    });
                    nteacherList.addAll(cteachers);

                    List<TNoticeClass> nclasses = n.getClasses();
                    nclasses.forEach(c -> {
                        c.setNoticeid(noticeid);
                        c.setCreateid(createid);
                        c.setCreatetime(createtime);
                    });
                    nclassList.addAll(nclasses);

                    TNoticeSchool ns = new TNoticeSchool();
                    ns.setNoticeid(noticeid);
                    ns.setSchoolid(schoolid);
                    ns.setCreateid(createid);
                    ns.setCreatetime(createtime);
                    schoolList.add(ns);
                });

                if (N_Utils.isListNotEmpty(nteacherList)) noticeTeacherMapper.insertList(teacherList);//通知教师关联信息
                if (N_Utils.isListNotEmpty(nclassList)) noticeClassMapper.insertList(classList);//通知班级关联信息
                if (N_Utils.isListNotEmpty(schoolList)) noticeSchoolMapper.insertList(schoolList);//通知学校关联信息
            }

            if (N_Utils.isListNotEmpty(msgList)) {//短信
                List<TMsg> msgs = new ArrayList<>();
                for (TMsg m : msgList) {
                    m.setCreatetime(createtime);
                    msgs.add(m);
                }
                if (N_Utils.isListNotEmpty(msgs)) {
                    msgMapper.insertList(msgs);//短信保存到数据库

                    //发短信
                    Integer pageSize = 2000;//手机号，多个手机号为用半角 , 分开，如13899999999,13688888888(最多2000个，必填)
                    Map<String, List<TMsg>> mcollect = msgs.stream().collect(Collectors.groupingBy(s -> s.getMsgtxt()
                            , LinkedHashMap::new, Collectors.toList()));
                    for (Map.Entry<String, List<TMsg>> entry : mcollect.entrySet()) {
                        List<TMsg> list = entry.getValue();
                        String msgtxt = list.get(0).getMsgtxt();
                        for (int i = 0; i < list.size(); i += pageSize) {
                            ZtMsgVo vo = new ZtMsgVo();
                            String userphone = list.stream().skip(i).limit(pageSize)
                                    .map(s -> s.getUserphone()).collect(Collectors.joining(","));
                            vo.setMobile(userphone);
                            vo.setContent(msgtxt);
                            SMSSender.sendSms(vo);
                        }
                    }
                }
            }
        } catch (Exception e) {
            log.error("批量添加通知和短信（发布考试用）失败", e.getMessage());
        }
    }
}
